﻿---------------------------------------------------------
--	Library
---------------------------------------------------------
local bzone = LibStub("LibBabble-Zone-3.0"):GetLookupTable()
local bboss = LibStub("LibBabble-Boss-3.0"):GetLookupTable()


---------------------------------------------------------
--	Localization
---------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("GridStatusRaidDebuff")
local LGrid = AceLibrary("AceLocale-2.2"):new("Grid")

L:RegisterTranslations("enUS", function()
	return {
	["Raid Debuff"] = true,
		["Option for %s"] = true,
		["Enable"] = true,
		["Enable %s"] = true,		
		["Icon Priority"] = true,
		["Color Priority"] = true,
		["Custom Color"] = true,
		["Color"] = true,
		colorDesc = "Modify Color",
		iconWarnName = "|cffff0000(No Icon)|r",
		iconWarnDesc = "\n\n|cffff0000You can't see this on center icon or other icon indicators. If you want to see this, you enable Raid Debuff of the border indicator or other color indicators.",		
		["Ignore dispellable debuff"] = true,		
		["Ignore undispellable debuff"] = true,		
	}
end)

L:RegisterTranslations("koKR", function()
	return {
		["Raid Debuff"] = "레이드 디버프",
		["Option for %s"] = "%s의 설정",
		["Enable"] = "활성화",
		["Enable %s"] = "%s 활성화",		
		["Icon Priority"] = "아이콘 우선순위",
		["Color Priority"] = "색상 우선순위",
		["Custom Color"] = "임의의 색상",
		["Color"] = "색상",
		colorDesc = "색상 변경",
		iconWarnName = "|cffff0000(아이콘 없음)|r",
		iconWarnDesc = "\n\n|cffff0000이 디버프는 중앙아이콘이나 아이콘 표시기로 볼수 없습니다. 디버프를 보기 위해서는 테두리나 다른 색상 표시기의 레이드 디버프를  활성화 해야됩니다.",
		["Ignore dispellable debuff"] = "해제 가능한 디버프를 무시합니다",		
		["Ignore undispellable debuff"] = "해제 불가능한 디버프를 무시합니다",		
	}
end)

L:RegisterTranslations("zhCN", function()
	return {
		["Raid Debuff"] = "团队减益",
		["Option for %s"] = "%s 设置",
		["Enable"] = "开启",
		["Enable %s"] = "开启 %s",		
		["Icon Priority"] = "Icon Priority",
		["Color Priority"] = "Color Priority",
		["Custom Color"] = "Custom Color",
		["Color"] = "Color",
		colorDesc = "Modify Color",
		iconWarnName = "|cffff0000(No Icon)|r",
		iconWarnDesc = "\n\n|cffff0000You can't see this on center icon or other icon indicators. If you want to see this, you enable Raid Debuff of the border indicator or other color indicators.",		
		["Ignore dispellable debuff"] = "Ignore dispellable debuff",		
		["Ignore undispellable debuff"] = "Ignore undispellable debuff",
}
end)

L:RegisterTranslations("zhTW", function()
	return {
		["Raid Debuff"] = "團隊減益",
		["Option for %s"] = "%s 選項",
		["Enable"] = "啟用",
		["Enable %s"] = "啟用%s",
		["Icon Priority"] = "Icon Priority",
		["Color Priority"] = "Color Priority",
		["Custom Color"] = "Custom Color",
		["Color"] = "Color",
		colorDesc = "Modify Color",
		iconWarnName = "|cffff0000(No Icon)|r",
		iconWarnDesc = "\n\n|cffff0000You can't see this on center icon or other icon indicators. If you want to see this, you enable Raid Debuff of the border indicator or other color indicators.",		
		["Ignore dispellable debuff"] = "Ignore dispellable debuff",		
		["Ignore undispellable debuff"] = "Ignore undispellable debuff",
	}
end)

L:RegisterTranslations("ruRU", function()
	return {
		["Raid Debuff"] = "Дебаффы рейда",
		["Option for %s"] = "Опции %s",
		["Enable"] = "Включить",
		["Enable %s"] = "Включить %s",
		["Icon Priority"] = "Icon Priority",
		["Color Priority"] = "Color Priority",
		["Custom Color"] = "Custom Color",
		["Color"] = "Color",
		colorDesc = "Modify Color",
		iconWarnName = "|cffff0000(No Icon)|r",
		iconWarnDesc = "\n\n|cffff0000You can't see this on center icon or other icon indicators. If you want to see this, you enable Raid Debuff of the border indicator or other color indicators.",		
		["Ignore dispellable debuff"] = "Ignore dispellable debuff",		
		["Ignore undispellable debuff"] = "Ignore undispellable debuff",
	}
end)

---------------------------------------------------------
--	local
---------------------------------------------------------
local zone
local db, myClass, myDispellable
local debuff_list = {}

local colorMap = {
	["Curse"] = { r = .6, g =  0, b = 1},
	["Magic"] = { r = .2, g = .6, b = 1},
	["Poison"] = {r =  0, g = .6, b =  0},
	["Disease"] = { r = .6, g = .4, b =  0},
}

local dispelMap = {
	["PRIEST"] = {["Magic"] = true, ["Disease"] = true},
	["PALADIN"] = {["Magic"] = true, ["Disease"] = true, ["Poison"] = true},
	["MAGE"] = {["Curse"] = true},
	["DRUID"] = {["Curse"] = true, ["Poison"] = true},
	["SHAMAN"] = {["Disease"] = true, ["Poison"] = true, ["Curse"] = (select(5,GetTalentInfo(3,18)) == 1)},	
}


---------------------------------------------------------
--	Core
---------------------------------------------------------
GridStatusRaidDebuff = GridStatus:NewModule("GridStatusRaidDebuff")

local GridStatusRaidDebuff = GridStatusRaidDebuff
GridStatusRaidDebuff.menuName = L["Raid Debuff"]

local GetSpellInfo = GetSpellInfo
local fmt = string.format

GridStatusRaidDebuff.defaultDB = {
	debug = false,	
	isFirst = true,	
	
	["alert_RaidDebuff"] = {
		text = L["Raid Debuff"],
		desc = L["Raid Debuff"],
		enable = true,
		color = { r = .0, g = .0, b = .0, a=1.0 },
		priority = 98,
		range = false,
	},
	
	ignDis = false,
	ignUndis = false,
	
	["debuff_options"] = {},	
}

function GridStatusRaidDebuff:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatuses()	
	db = self.db.profile.debuff_options	
end

function GridStatusRaidDebuff:OnEnable()
	self.debugging = self.db.profile.debug	
	
	myClass = select(2, UnitClass("player"))
	myDispellable = dispelMap[myClass]
	
	if self.db.profile.isFirst then		
		GridFrame.db.profile.statusmap["icon"].alert_RaidDebuff  =  true,
		GridFrame:UpdateAllFrames()
		self.db.profile.isFirst = false
	end

	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneCheck")
	
end

function GridStatusRaidDebuff:Reset()
	self.super.Reset(self)
	self:UnregisterStatuses()
	self:RegisterStatuses()
end

function GridStatusRaidDebuff:PLAYER_ENTERING_WORLD()	
	self:ZoneCheck()	
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")	
end

function GridStatusRaidDebuff:ZoneCheck()
	zone = GetRealZoneText()

	self:UpdateAllUnit()

	if debuff_list[zone] then
		if not self:IsEventRegistered("UNIT_AURA") then				
			self:RegisterEvent("UNIT_AURA", "ScanUnit")
			self:RegisterEvent("Grid_UnitJoined")			
			self:RegisterEvent("PLAYER_TALENT_UPDATE")
		end
	else			
		if self:IsEventRegistered("UNIT_AURA") then				
			self:UnregisterEvent("UNIT_AURA")
			self:UnregisterEvent("Grid_UnitJoined")	
			self:UnregisterEvent("PLAYER_TALENT_UPDATE")
		end
	end
end

function GridStatusRaidDebuff:RegisterStatuses()
	self:RegisterStatus("alert_RaidDebuff", L["Raid Debuff"])
	self:CreateMainMenu()
end

function GridStatusRaidDebuff:UnregisterStatuses()
	self:UnregisterStatus("alert_RaidDebuff")
end

function GridStatusRaidDebuff:Grid_UnitJoined(guid, unitid)
	self:ScanUnit(unitid)
end

function GridStatusRaidDebuff:PLAYER_TALENT_UPDATE()
	if myClass == "SHAMAN" then
		myDispellable["Curse"] = (select(5,GetTalentInfo(3,18)) == 1)
	end
end

-- Create a custom tooltip for debuff description
local tip = CreateFrame("GameTooltip", "GridStatusRaidDebuffTooltip", nil, "GameTooltipTemplate")
tip:SetOwner(UIParent, "ANCHOR_NONE")
for i = 1, 10 do
	tip[i] = _G["GridStatusRaidDebuffTooltipTextLeft"..i]
	if not tip[i] then
		tip[i] = tip:CreateFontString()
		tip:AddFontStrings(tip[i], tip:CreateFontString())
	end
end

function GridStatusRaidDebuff:UpdateAllUnit()
	for guid, unitid in GridRoster:IterateRoster() do
		self:ScanUnit(unitid)
	end
end

function GridStatusRaidDebuff:ScanUnit(unitid)
	local guid = UnitGUID(unitid)	
	if not GridRoster:IsGUIDInRaid(guid) then
		return
	end
	self:Debug("UNIT_AURA", unitid, guid)
	
	local name, rank, icon, count, debuffType, duration, expirationTime, caster, isStealable	
	local settings = self.db.profile["alert_RaidDebuff"]	
	
	if (settings.enable and debuff_list[zone]) then	
		local d_name, di_prior, dc_prior, dt_prior, d_icon,d_color,d_startTime,d_durTime,d_count
		local data
		
		di_prior = 0
		dc_prior = 0
		dt_prior = 0
		
		local index = 0
		while true do
			index = index + 1
			name, _, icon, count, debuffType, duration, expirationTime, caster, isStealable = UnitAura(unitid, index, "HARMFUL")

			if not name then
				break
			end

			if debuff_list[zone][name] then		
				data = debuff_list[zone][name]
				
				if not data.disable and
				   not (self.db.profile.ignDis and myDispellable[debuffType]) and
				   not (self.db.profile.ignUndis and debuffType and not myDispellable[debuffType]) then
				
					if di_prior < data.i_prior then
						di_prior = data.i_prior
						d_name = name
						d_icon = 	not data.noicon and icon
						if data.timer and dt_prior < data.i_prior then
							d_startTime = expirationTime - duration
							d_durTime = duration
						end
					end				
					--Stack
					if data.stackable then
						d_count = count
					end
					--Color Priority
					if dc_prior < data.c_prior then
						dc_prior = data.c_prior					
						d_color = (data.custom_color and data.color) or colorMap[debuffType] or settings.color					
					end
				end
			else
			--Register New Debuff
			end
		end
		
		if d_color and not d_color.a then
			d_color.a = settings.color.a
		end
		
		if d_color and d_color.a == 0 then			
			d_color.a = 1
		end
			
		if d_name then
			self.core:SendStatusGained(
			guid, "alert_RaidDebuff", settings.priority, (settings.range and 40),
			d_color, nil, nil, nil, d_icon, d_startTime, d_durTime, d_count)
		else
			self.core:SendStatusLost(guid, "alert_RaidDebuff")	
		end
	else
		self.core:SendStatusLost(guid, "alert_RaidDebuff")	
	end
end

---------------------------------------------------------
--	For External
---------------------------------------------------------
local function getDb(zone, name, arg, ret)	
	if db[zone] and db[zone][name] and db[zone][name][arg] ~= nil then
		return db[zone][name][arg]
	end
	return ret
end

local function insertDb(zone, name, arg, value)
	if not db[zone] then db[zone] = {} end
	if not db[zone][name] then db[zone][name] = {} end
	
	if arg then
		db[zone][name][arg] = value
	end
end

function GridStatusRaidDebuff:Debuff(en_zone, debuffID, order, icon_priority, color_priority, timer, stackable, color, default_disable, noicon)
	local zone = bzone[en_zone]
	local name, _, icon = GetSpellInfo(debuffID)		
	local args, data
	local settings = self.db.profile["alert_RaidDebuff"]
	
	self:CreateZoneMenu(zone)	
	
	if not debuff_list[zone][name] then
		debuff_list[zone][name] = {}		
		data = debuff_list[zone][name]		
		
		local description = L["Enable %s"]:format(name or debuffID)		
		tip:SetHyperlink("spell:"..debuffID)
		if tip:NumLines() > 1 then
			description = tip[tip:NumLines()]:GetText()
		end
		
		local menuName = name		
		
		if noicon then
			menuName = fmt("|T%s:0|t%s%s", "Interface\\Icons\\mail_gmicon",menuName, L["iconWarnName"])
			description = fmt("%s%s", description, L["iconWarnDesc"])
		else
			menuName = fmt("|T%s:0|t%s", icon, menuName)
		end
		
		args = self.options.args[zone].args		
		
		args[name] = {
			type = "group",
			name = menuName,
			desc = description,
			order = order,
			args = {
				["enable"] = {
					type = "toggle",
					name = L["Enable"],
					desc = L["Enable %s"]:format(name),
					order = 1,
					get = function()
									return not getDb(zone,name,"disable",default_disable)
								end,
					set = function(v)
									insertDb(zone,name,"disable",not v)									
									data.disable = not v
									self:UpdateAllUnit()
								end,
				},
				
				["icon priority"] = {
					type = "range",
					name = L["Icon Priority"],
					desc = L["Option for %s"]:format(L["Icon Priority"]),
					order = 2,
					min = 1,
					max = 10,
					step = 1,
					get = function()									
									return getDb(zone,name,"i_prior",icon_priority)
								end,
					set = function(v)
									insertDb(zone,name,"i_prior",v)
									data.i_prior = v
								end,	
				},
				["color priority"] = {
					type = "range",
					name = L["Color Priority"],
					desc = L["Option for %s"]:format(L["Color Priority"]),
					order = 3,
					min = 1,
					max = 10,
					step = 1,
					get = function() 
									return getDb(zone,name,"c_prior",color_priority)
								end,
					set = function(v)
									insertDb(zone,name,"c_prior",v)
									data.c_prior = v
								end,					
				},
				["custom color"] = {
					type = "toggle",
					name = L["Custom Color"],
					desc = L["Enable %s"]:format(L["Custom Color"]),
					order = 4,
					get = function() 
									return getDb(zone,name,"custom_color",color ~= nil)
								end,
					set = function(v)
									insertDb(zone,name,"custom_color",v)
									data.custom_color = v
									
									if v then
										insertDb(zone,name,"color", settings.color)
										data.color = settings.color
									end
								end,					
				},
				["color"] = {
					type = "color",
					name = L["Color"],
					desc = L["Option for %s"]:format(L["Color"]),
					order = 5,
					disabled = function()
											return not getDb(zone,name,"custom_color",color ~= nil)
										end,	
					hasAlpha = false,
					get = function ()									
									t = getDb(zone,name,"color", color or {r = 1, g = 0, b = 0})
									return t.r, t.g, t.b
							  end,
					set = function (ir, ig, ib)
									local t = {r = ir, g = ig, b = ib}
									insertDb(zone,name,"color",t)
								  data.color = t
							  end,
				}			
			},
		}
	end
	
	data.disable = getDb(zone,name,"disable",default_disable)
	data.i_prior = getDb(zone,name,"i_prior",icon_priority)
	data.c_prior = getDb(zone,name,"c_prior",color_priority)
	data.custom_color = getDb(zone,name,"custom_color",color ~= nil)
	data.color = getDb(zone,name,"color",color)
	data.stackable = getDb(zone,name,"stackable",stackable)
	data.timer = getDb(zone,name,"timer",timer)
	data.noicon = getDb(zone,name,"noicon",noicon)
end


function GridStatusRaidDebuff:BossName(en_zone, order, en_boss)
	local zone = bzone[en_zone]
	local boss = bboss[en_boss]	
	
	self:CreateZoneMenu(zone)
	
	local args = self.options.args
	
	args[zone].args[boss] = {		
			type = "group",
			name = fmt("%s%s%s","   [ ", boss," ]"),
			order = order,
			args = {}
	}
end

function GridStatusRaidDebuff:CreateZoneMenu(zone)
	local args
	if not debuff_list[zone] then 
		debuff_list[zone] = {}
				
		args = self.options.args
					
		args[zone] = {
			type = "group",
			name = zone,
			desc = L["Option for %s"]:format(zone),			
			args = {},
		}
	end
end

function GridStatusRaidDebuff:CreateMainMenu()
	local args = self.options.args	
	
	for i,k in pairs(args["alert_RaidDebuff"].args) do
		args[i] = k		
	end	
	
	args["alert_RaidDebuff"].hidden = true
	
	args["Border"] = {
			type = "toggle",
			name = LGrid["Border"],			
			desc = L["Enable %s"]:format(LGrid["Border"]),
			order = 98,
			disabled = InCombatLockdown,
			get = function() return GridFrame.db.profile.statusmap["border"].alert_RaidDebuff end,
			set = function(v) 
							GridFrame.db.profile.statusmap["border"].alert_RaidDebuff  =  v 
							GridFrame:UpdateAllFrames()
						end,
	}
	
	args["Icon"] = {
			type = "toggle",
			name = LGrid["Center Icon"],			
			desc = L["Enable %s"]:format(LGrid["Center Icon"]),
			order = 99,
			disabled = InCombatLockdown,
			get = function() return GridFrame.db.profile.statusmap["icon"].alert_RaidDebuff end,
			set = function(v) 
							GridFrame.db.profile.statusmap["icon"].alert_RaidDebuff  =  v 
							GridFrame:UpdateAllFrames()
						end,
	}
	
	args["Ignore dispellable"] = {
		type = "toggle",
		name = L["Ignore dispellable debuff"],
		desc = L["Ignore dispellable debuff"],
		order = 100,
		get = function() return self.options.args.ignDis end,
		set = function(v) 
						self.options.args.ignDis = v
						self:UpdateAllUnit()
					end,
	
	}
	
	args["Ignore undispellable"] = {
		type = "toggle",
		name = L["Ignore undispellable debuff"],
		desc = L["Ignore undispellable debuff"],
		order = 101,
		get = function() return self.options.args.ignUndis end,
		set = function(v) 
						self.options.args.ignUndis = v
						self:UpdateAllUnit()
					end,	
	}
end